/*  LAPcan driver kernel module for Linux */
/*  Copyright (C) 2003 Kvaser AB - support@kvaser.com - www.kvaser.com  */

#ifndef _LAPCAN_H_
#define _LAPCAN_H_

#include "canIfData.h"
#include "lapcmds.h"

///////////////////////////////////////////////////////////////////////
// defines
//

#define LP_FIFO_SIZE 64
#define MAIN_RCV_BUF_SIZE 16
#define FILE_RCV_BUF_SIZE 500
#define TX_CMD_BUF_SIZE 16
#define TX_CHAN_BUF_SIZE 500
#define CAN_MAJOR 91
#define DEVICE_NAME_STRING "lapcan_cs"
#define NR_CHANNELS 2
#define LAPCAN_TX_BUF_SIZE 200

/*====================================================================
 data structures
====================================================================*/

/* Used to filter incoming messages */
typedef struct FilterNode
{
  unsigned int cmdNr;
  int cmdNrMask;
  unsigned char chanNr;
  char chanNrMask;
  unsigned char flags;
  char flagsMask;
  unsigned char isPass;
  struct FilterNode *next;
} FilterNode;

//struct LapcanOpenFileNode;

/* Everyone who opens the driver gets one of these */
typedef struct LapcanOpenFileNode
{
  int rcvBufHead;
  int rcvBufTail;
  lpcCmd fileRcvBuffer [FILE_RCV_BUF_SIZE];
  wait_queue_head_t rxWaitQ;
  struct file *filp;
  struct LocalInfo *localPtr;
  int chanNr;
  unsigned char writeIsBlock;
  unsigned char readIsBlock;
  long writeTimeout;
  long readTimeout;
  FilterNode *msgFilter;
  struct LapcanOpenFileNode *next;
  unsigned long overruns [NR_CHANNELS];

} LapcanOpenFileNode;

// structure for the hardware context

typedef struct 
{
  unsigned           portAddress;
  unsigned char      isPresent;
  unsigned char      isInit;

  volatile int       recIsBusy;
  /* Processes waiting for all messages to be sent */
  wait_queue_head_t  flushQ [NR_CHANNELS];
  /* Number of messages remaining in the sendbuffers */ 
  int sentTXflagCount [NR_CHANNELS];
  int recvTXflagCount [NR_CHANNELS];

  // usage counter for Lapcan TX FIFO
  volatile unsigned char      fifoCount;

  // Buffer structures
  unsigned char      curCmdLen;
  unsigned char      curCmdPos;
  lpcCmd          mainRcvBuffer   [MAIN_RCV_BUF_SIZE];
  int                mainRcvBufHead; /* Where we write incoming messages */
  int                mainRcvBufTail; /* Where we read incoming messages  */

  lpcCmd          txCmdBuffer     [TX_CMD_BUF_SIZE]; /* Control messages */
  unsigned int       txCmdBufHead;   /* Where we write outgoing control messages */
  unsigned int       txCmdBufTail;   /* The messages are sent from this end */
  wait_queue_head_t  txCmdWaitQ;    /* WaitQ for sending commands */

  /* These are the outgoing channelqueues */
  cmdTxExtMessage    txChanBuffer    [NR_CHANNELS] [TX_CHAN_BUF_SIZE];
  unsigned int       txChanBufHead   [NR_CHANNELS];
  unsigned int       txChanBufTail   [NR_CHANNELS];
  wait_queue_head_t  txChanWaitQ    [NR_CHANNELS]; 
  unsigned char      txQChipFull     [NR_CHANNELS]; /* Used for flow control */
  /* Data to keep track of who sent a particular message */
#ifdef MULTI_OPEN 
  struct file        *unsentIdBuffer [NR_CHANNELS] [ID_BUFFER_SIZE];
  unsigned int       idBufferHead    [NR_CHANNELS];
  unsigned int       idBufferTail    [NR_CHANNELS];
#endif
  LapcanOpenFileNode *openFileList;
  struct tq_struct   txTaskQ;
  /* Not used for now */
  //struct tq_struct   rxTaskQ;
  
  // Some data about the card; these are obtained when initializing the card.
  unsigned char      hwInfoIsValid;
  unsigned char      swInfoIsValid;
  unsigned char      useDCD;
  unsigned char      useOUT1Toggle;
  unsigned int       hwRevision;
  unsigned int       swRevision;
  unsigned int       swOptions;
  unsigned int       clockResolution;

  struct file        *exclusiveFlagPtr    [NR_CHANNELS];
  unsigned short     channelOpenCount  [NR_CHANNELS];
  /* Lock while modifying exclusive & open flags */
  spinlock_t         exclAndOpenLock [NR_CHANNELS];
  spinlock_t         sendQLock [NR_CHANNELS];

} LapcanContext;

typedef struct LocalInfo 
{
  dev_link_t		link;
  dev_node_t		node1;
  LapcanContext         ctx;
  int			stop;
  struct bus_operations *bus;
} LocalInfo;

#endif


