

/*
**                        Copyright 1997 by KVASER AB
**            P.O Box 4076 S-51104 KINNAHULT, SWEDEN Tel. +46 320 15287
**
** This software is furnished under a license and may be used and copied
** only in accordance with the terms of such license.
**
**  $Revision: 1.18 $
**  $Header: /home/cvs/beta/linuxcan/include/pcican.h,v 1.18 2003/04/07 09:02:03 mb Exp $
** File: 
**   pcican.h
** Project:
**   Pcican Linux
** Desciption: 
**   Pcican module datastructures.
*/

#ifndef _PCICAN_H_
#define _PCICAN_H_

#include "canIfData.h"
#include "vcanevt.h"
#include "dallas.h"

/*****************************************************************************/
/* defines */
/*****************************************************************************/

#define MAIN_RCV_BUF_SIZE 16
#define FILE_RCV_BUF_SIZE 500
#define TX_CHAN_BUF_SIZE 500
#define DEVICE_NAME_STRING "pcican"
#define MAX_CHANNELS 4
#define PCICAN_MAX_DEV 16
#define PCICAN_VENDOR  0x10e8
#define PCICAN_ID 0x8406

// Standard value: Pushpull  (OCTP1|OCTN1|OCTP0|OCTN0|OCM1)
#define OCR_DEFAULT_STD 0xDA
// For Galathea piggyback.
#define OCR_DEFAULT_GAL 0xDB


#define MAX_ERROR_COUNT 128
#define ERROR_RATE 30000
#define PCICAN_BYTES_PER_CIRCUIT 0x20

/*****************************************************************************/
/* Xilinx                                                                    */
/*****************************************************************************/

//
// These register values are valid for revision 14 of the Xilinx logic.
//
#define XILINX_OUTA         0   // Bit 7 used to write bits to serial memory DS2430.
#define XILINX_INA          1   // Bit 7 used to read back bits from serial memory DS2430.
#define XILINX_CTRLA        2   // Sets the function of the Xilinx pins normally set to zero.
#define XILINX_UNUSED       3
#define XILINX_OUTB         4   // Bit 7 used to reset the SJA-1000.
#define XILINX_INB          5   // Bit 7 used to read back the reset line value.
#define XILINX_CTRLB        6   // Sets the function of the Xilinx pins normally set to zero.
#define XILINX_VERINT       7   // Lower nibble simulate interrupts, high nibble version number.

#define XILINX_PRESUMED_VERSION     14

/*****************************************************************************/
/*  From canlib30/Src/CANLIB32/2K/include/vcanio.h                           */
/*****************************************************************************/

#define CAN_EXT_MSG_ID          0x80000000

#define CAN_BUSSTAT_BUSOFF              0x01
#define CAN_BUSSTAT_ERROR_PASSIVE       0x02
#define CAN_BUSSTAT_ERROR_WARNING       0x04
#define CAN_BUSSTAT_ERROR_ACTIVE        0x08
#define CAN_BUSSTAT_BUSOFF_RECOVERY     0x10
#define CAN_BUSSTAT_IGNORING_ERRORS     0x20

#define CAN_CHIP_TYPE_UNKNOWN           0
#define CAN_CHIP_TYPE_VIRTUAL           1
#define CAN_CHIP_TYPE_SJA1000           2
#define CAN_CHIP_TYPE_527               3
#define CAN_CHIP_TYPE_C200              4


/*****************************************************************************/
/*  data structures                                                          */
/*****************************************************************************/

typedef union {
    unsigned long L;
    struct { unsigned short w0, w1; } W;
    struct { unsigned char b0, b1, b2, b3; } B;
} WL;

typedef struct s_sja1000_state {
  int state;  /* buson / busoff / error passive / warning */
  int txerr;  /* tx error counter on sja1000 */
  int rxerr;  /* rx error counter on sja1000 */
} SJA1000_STATE;

/* File pointer specific data */
typedef struct PciCanOpenFileNode
{
    int rcvBufHead;
    int rcvBufTail;
    VCAN_EVENT fileRcvBuffer [FILE_RCV_BUF_SIZE];
    unsigned char transId;
    wait_queue_head_t rxWaitQ;
    struct file *filp;
    struct PciCanChanData *chanData;
    int chanNr;
    unsigned char writeIsBlock;
    unsigned char readIsBlock;
    unsigned char modeTx;
    unsigned char modeTxRq;
    unsigned char channelOpen;
    unsigned char channelLocked;    
    long writeTimeout;
    long readTimeout;
    VCanMsgFilter filter;
    struct PciCanOpenFileNode *next;
    unsigned long overruns;

} PciCanOpenFileNode;

/* Channel specific data */
typedef struct PciCanChanData
{
    /* Ports and addresses */
    unsigned           sja1000;
    unsigned           xilinxAddressOut;
    unsigned           xilinxAddressCtrl;
    unsigned           xilinxAddressIn;
    unsigned char      channel;
    unsigned char      chipType;
    unsigned char      ean[6];
    unsigned long      serialHigh;
    unsigned long      serialLow;

    /* Status */
    unsigned char      isOnBus;
    unsigned char      transType;   // TRANSCEIVER_TYPE_xxx
    unsigned char      lineMode;    // TRANSCEIVER_LINEMODE_xxx
    unsigned char      resNet;      // TRANSCEIVER_RESNET_xxx
    unsigned char      chipState;
    unsigned char      transId;
    unsigned int       overrun;
    unsigned int       errorCount;
    unsigned long      errorTime;
    DALLAS_CONTEXT     chanEeprom;    
    SJA1000_STATE      sjaState;

    int                minorNr;

    CAN_MSG               txChanBuffer[TX_CHAN_BUF_SIZE];
    CAN_MSG               *currentTxMsg;
    
    volatile unsigned int txChanBufHead;
    volatile unsigned int txChanBufTail;
    wait_queue_head_t     txChanWaitQ; 
    /* Processes waiting for all messages to be sent */
    wait_queue_head_t  flushQ;

    struct PciCanOpenFileNode *openFileList;
    struct tq_struct   txTaskQ;

    unsigned short     fileOpenCount;
    unsigned short     chanOpenCount;
    unsigned long      usPerTick;

    spinlock_t         openLock;
    spinlock_t         sendQLock;

} PciCanChanData;

/*  Cards specific data */
typedef struct PciCanCardData {
    unsigned int       nrChannels;
    DALLAS_CONTEXT     cardEeprom;
    unsigned long      serialNumber;
    unsigned char      ean[6];
    unsigned char      firmwareVersionMajor;
    unsigned char      firmwareVersionMinor;
    unsigned char      firmwareVersionBuild;
    unsigned char      isGalathea;

    /* Ports and addresses */
    unsigned           sjaBase;
    unsigned           xilinx;
    int                irq;
    unsigned           pciIf;
    unsigned char      cardPresent;
    PciCanChanData     chanData[MAX_CHANNELS];
    struct PciCanCardData *next;
} PciCanCardData;

typedef struct PciCanDriverData {
    int minorNr;
    int majorDevNr;
    struct timeval startTime;
} PciCanDriverData;

/*****************************************************************************/
/*  Function definitions                                                     */
/*****************************************************************************/

static int pcicanOpen (struct inode *inode, struct file *filp);
static int pcicanClose (struct inode *inode, struct file *filp);
static int pcicanIOCtl (struct inode *inode, struct file *filp, 
                 unsigned int cmd, unsigned long arg);

static int pciCanTransmitMessage (PciCanChanData *chd, CAN_MSG *m);
static void pcicanSend (void *void_chanData);
static int pciCanBusOn (PciCanChanData *chd);
static int pciCanBusOff (PciCanChanData *chd);
static int pciCanSetBusParams (PciCanChanData *chd, VCanBusParams *par);
static int pciCanSetOutputMode (PciCanChanData *chd, int silent);
static int pciCanRequestChipState (PciCanChanData *chd);
static int pciCanGetBusParams (PciCanChanData *chd, VCanBusParams *par);


#endif /* _PCICAN_H_ */


