/* Kvaser CAN driver PCIcan hardware specific parts                    */
/* Older version of PCIcan driver                                      */
/* Copyright (C) 2003 Kvaser AB - support@kvaser.com - www.kvaser.com  */

/* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or 
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
* 
*/

#include <linux/config.h>
#include <linux/pci.h>

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/ptrace.h>

//#include <linux/malloc.h>
#include <linux/slab.h>

#include <linux/string.h>
#include <linux/timer.h>
#include <linux/tqueue.h>
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <linux/ioport.h>
#include <linux/proc_fs.h>

#include <asm/io.h>
#include <asm/system.h>
#include <asm/bitops.h>
#include <asm/uaccess.h>

/* Module versioning */
#define EXPORT_SYMTAB
#include <linux/module.h>

/* retrieve the CONFIG_* macros */
#include <linux/autoconf.h>
#if defined(CONFIG_MODVERSIONS) && !defined(MODVERSIONS)
#   define MODVERSIONS
#endif

#ifdef MODVERSIONS
#   include <linux/modversions.h>
#endif

#define put_user_ret(x,ptr,ret) ({ if (put_user(x,ptr)) return ret; })
#define get_user_ret(x,ptr,ret) ({ if (get_user(x,ptr)) return ret; })
#define copy_to_user_ret(to,from,n,retval) ({ if (copy_to_user(to,from,n)) return retval; })
#define copy_from_user_ret(to,from,n,retval) ({ if (copy_from_user(to,from,n)) return retval; })

//#include <sysdep.h> /* Some compatibility with older kernels (qqq)*/

/* Kvaser definitions */
#include "dallas.h"
#include "vcanevt.h"
#include "pcican.h"
#include "vcan_ioctl.h"

#include "sja1000.h"
#include "amcc5920.h"
#include "hwnames.h"


/* Parameters that can be set with 'insmod' */

/* The old way: bit map of interrupts to choose from */
/* This means pick from 15, 14, 12, 11, 10, 9, 7, 5, 4, and 3 */
//static u_int irq_mask = 0xdeb8;
/* Newer, simpler way of listing specific interrupts */
// static int irq_list[4] = { -1 };
 


//QQQ MODULE_PARM(irq_mask, "i");
//qqq MODULE_PARM(irq_list, "1-4i");
MODULE_LICENSE("GPL");

PciCanDriverData driverData;

/*
If you do not define PCICAN_DEBUG at all, all the debug code will be
left out.  If you compile with PCICAN_DEBUG=0, the debug code will
be present but disabled -- but it can then be enabled for specific
modules at load time with a 'debug_level=#' option to insmod.
i.e. >insmod kvpcican debug_level=#
*/

#ifdef PCICAN_DEBUG
    static int debug_level = PCICAN_DEBUG;
    MODULE_PARM(debug_level, "i");
#   define DEBUG(n, args...) if (debug_level>=(n)) printk("<" #n ">" args)

    char *version =
                   "Kvaser PCIcan Driver Version 1.0.0";
#else
#   define DEBUG(n, args...)
#endif


static char *device_name = DEVICE_NAME_STRING;
PciCanCardData *pciCanCards = NULL;


/*======================================================================*/
/* file operations...                                                   */
/*======================================================================*/

struct file_operations pcicanFOps = {
llseek:    NULL,        
read:      NULL,        
write:     NULL,        
readdir:   NULL,        
poll:      NULL,        
ioctl:     pcicanIOCtl, 
mmap:      NULL,        
open:      pcicanOpen,  
flush:     NULL,        
release:   pcicanClose, 
fsync:     NULL        
           /* Fills rest with NULL entries */
};

/*======================================================================*/
/* /proc read function                                                  */
/*======================================================================*/

int  pciCanProcRead (char *buf, char **start, off_t offset,
                     int count, int *eof, void *data)
{
    int len = 0;
    //int limit = count - 80; /* Don't print more than this */
    
    len += sprintf(buf+len,"\ntotal channels %d\n", driverData.minorNr);
    
    *eof = 1;
    return len;
}


/*======================================================================*/
/* Time                                                                 */
/*======================================================================*/

static unsigned long pciCanTime(int usPerTick)
{
    struct timeval tv;
    do_gettimeofday(&tv);
    
    tv.tv_usec -= driverData.startTime.tv_usec;
    tv.tv_sec -= driverData.startTime.tv_sec;
    
    return tv.tv_usec / usPerTick + (1000000/usPerTick) * tv.tv_sec;
}


/*======================================================================*/
/*  Can we send now?                                                    */
/*======================================================================*/

static int pciCanTxAvailable (PciCanChanData *chd)
{
    unsigned circAddr = chd->sja1000;
    return ((inb(circAddr + PCAN_SR) & PCAN_TBS) == PCAN_TBS);
} /* pciCanTxAvailable */

/*======================================================================*/
/*  Calculate queue length                                              */
/*======================================================================*/

inline static int getQLen(int head, int tail, int size)
{
    return (head < tail ? head - tail + size : head - tail);
}

/*======================================================================*/
/*  Discard send queue                                                  */
/*======================================================================*/

static void vCanFlushSendBuffer (PciCanChanData *chd)
{
    spin_lock(&chd->sendQLock);
    chd->txChanBufHead = 0;
    chd->txChanBufTail = 0;
    spin_unlock(&chd->sendQLock);
}

/*======================================================================*/
/*  Deliver to receive queue                                            */
/*======================================================================*/

int pciCanDispatchEvent(PciCanChanData *chd, VCAN_EVENT *e)
{ 
    PciCanOpenFileNode *fileNodePtr;
    int rcvQLen;
    
    /* Update and notify readers */
    for (fileNodePtr = chd->openFileList; fileNodePtr != NULL; fileNodePtr = fileNodePtr->next){
        
        /* Event filter */
        if (!(e->tag & fileNodePtr->filter.eventMask)) continue;
        
        if (e->tag == V_RECEIVE_MSG && ((CAN_MSG*)e)->flags & VCAN_MSG_FLAG_TXACK){
            /* Skip if we sent it ourselves and we dont want the ack */
            if (e->transId == fileNodePtr->transId && !fileNodePtr->modeTx) continue;
            
        }
        
        if (e->tag == V_TRANSMIT_MSG && ((CAN_MSG*)e)->flags & VCAN_MSG_FLAG_TXRQ){
            /* Receive only if we sent it and we want the tx request */
            if (e->transId != fileNodePtr->transId || !fileNodePtr->modeTxRq) continue;
        }
        
        /* CAN filter */
        if (e->tag == V_RECEIVE_MSG || e->tag == V_TRANSMIT_MSG){
            unsigned int id = e->tagData.msg.id & ~VCAN_EXT_MSG_ID;
            if ((e->tagData.msg.id & VCAN_EXT_MSG_ID) == 0) {
                /* Standard message */
                if ((fileNodePtr->filter.stdId  ^ id) & fileNodePtr->filter.stdMask) {
                    continue;
                }
            } else {
                /* Extended message */
                if ((fileNodePtr->filter.extId  ^ id) & fileNodePtr->filter.extMask) continue;
            }
            
            //	    /* Filter on message flags ; note inverting */
            //if (!((fileNodePtr->filter.msgFlags ^ e->tagData.msg.flags) & fileNodePtr->filter.flagsMask)) continue;
            
            if (e->tagData.msg.flags & VCAN_MSG_FLAG_OVERRUN) fileNodePtr->overruns++;
        }
        
        rcvQLen = getQLen(fileNodePtr->rcvBufHead, 
            fileNodePtr->rcvBufTail, FILE_RCV_BUF_SIZE);
        
        if (rcvQLen >= FILE_RCV_BUF_SIZE - 1){
            /* The buffer is full, ignore new message */
            fileNodePtr->overruns++;
            /* Mark message that arrived before this one */
            {
                int i = FILE_RCV_BUF_SIZE - 1, head = fileNodePtr->rcvBufHead;
                for (i = 0; i; --i){
                    head = (head == 0 ? FILE_RCV_BUF_SIZE - 1 : head - 1);
                    if (fileNodePtr->fileRcvBuffer[head].tag == V_RECEIVE_MSG) break;
                }
                if (i) fileNodePtr->fileRcvBuffer[head].tagData.msg.flags |= VCAN_MSG_FLAG_OVERRUN;		
            }
            
        } else {
            memcpy(&(fileNodePtr->fileRcvBuffer[fileNodePtr->rcvBufHead]), e, sizeof(VCAN_EVENT));
            if (++fileNodePtr->rcvBufHead >= FILE_RCV_BUF_SIZE) fileNodePtr->rcvBufHead = 0;
            /* Wake up if the queue was empty before */
            if (rcvQLen == 0) {
                wake_up_interruptible(&fileNodePtr->rxWaitQ);
            }
        }
    }
    return 0;
}

/*======================================================================*/
/*    Open - File operation                                             */
/*======================================================================*/

int pcicanOpen(struct inode *inode, struct file *filp)
{
    struct PciCanOpenFileNode *openFileNodePtr;
    PciCanCardData *cardData = NULL;
    PciCanChanData *chanData = NULL;
    
    int minorNr;
    int channelNr;
    
    minorNr = MINOR(inode->i_rdev);
    if (pciCanCards == NULL) return -ENODEV;
    
    /* Find the right minor inode number */
    for (cardData = pciCanCards; cardData != NULL; cardData = cardData->next){
        for (channelNr = 0; channelNr < cardData->nrChannels; channelNr++){
            if (minorNr == cardData->chanData[channelNr].minorNr){
                chanData = &cardData->chanData[channelNr];
                break;
            }
        }
    }
    
    if (chanData == NULL) return -ENODEV;
    
    /* The open-field holds the usage count for this pcican*/
    
    /* Allocate memory and zero the whole struct */
    openFileNodePtr = kmalloc(sizeof(PciCanOpenFileNode), GFP_KERNEL);  
    if (!openFileNodePtr) return -ENOMEM;
    memset(openFileNodePtr, 0, sizeof(PciCanOpenFileNode));
    
    /* Init wait queue */
    init_waitqueue_head (&(openFileNodePtr->rxWaitQ));
    
    openFileNodePtr->filp = filp;
    openFileNodePtr->chanData = chanData;
    openFileNodePtr->writeIsBlock = 1;
    openFileNodePtr->readIsBlock = 1;
    openFileNodePtr->chanNr = -1;
    openFileNodePtr->writeTimeout = -1;
    openFileNodePtr->readTimeout = -1;
    openFileNodePtr->transId = chanData->transId++;
    openFileNodePtr->filter.eventMask = ~0;
    
    /* Insert this node first in list of "opens" */
    spin_lock(&chanData->openLock );
    openFileNodePtr->next = chanData->openFileList;
    chanData->openFileList = openFileNodePtr;
    chanData->fileOpenCount++;
    spin_unlock(&chanData->openLock );
    
    /* We want a pointer to the node as private_data */
    filp->private_data = openFileNodePtr;
    
    MOD_INC_USE_COUNT;
    
    return 0; 
}
/*======================================================================*/
/*    Release - File operation                                          */
/*======================================================================*/

int pcicanClose (struct inode *inode, struct file *filp) {
    
    struct PciCanOpenFileNode **openPtrPtr;
    struct PciCanOpenFileNode *fileNodePtr;
    PciCanChanData *chanData;
    
    fileNodePtr = filp->private_data;
    chanData = fileNodePtr->chanData;
    /* Decrease usage count for this pcican */
    
    spin_lock(&chanData->openLock);
    /* Find the open file node */
    openPtrPtr = &chanData->openFileList;
    for(;*openPtrPtr != NULL; openPtrPtr = &((*openPtrPtr)->next)) {
        if ((*openPtrPtr)->filp == filp) break;
    }
    /* We did not find anything? */
    if (*openPtrPtr == NULL) {
        spin_unlock(&chd->openLock);
        return -EBADF;
    }
    /* openPtrPtr now points to the next-pointer that points to the correct node */
    fileNodePtr = *openPtrPtr; 
    
    chanData->fileOpenCount--;
    
    /* Remove node */
    *openPtrPtr = (*openPtrPtr)->next;
    spin_unlock(&chd->openLock);
    kfree(fileNodePtr);
    
    if (!chanData->fileOpenCount) {
        pciCanBusOff(chanData);
        vCanFlushSendBuffer(chanData);
    }
    
    MOD_DEC_USE_COUNT;
    
    return 0;
}

int txQFull(PciCanChanData *chd) 
{
    return (getQLen(chd->txChanBufHead, chd->txChanBufTail, TX_CHAN_BUF_SIZE) >= TX_CHAN_BUF_SIZE - 1);
}

int rxQEmpty(PciCanOpenFileNode *fileNodePtr) 
{
    return (getQLen(fileNodePtr->rcvBufHead, fileNodePtr->rcvBufTail, FILE_RCV_BUF_SIZE) == 0);
}


/*======================================================================*/
/*  IOCtl - File operation                                              */
/*  This function is not reentrant with the same file descriptor!       */
/*======================================================================*/

int pcicanIOCtl (struct inode *inode, struct file *filp, 
                 unsigned int ioctl_cmd, unsigned long arg)
{
    PciCanOpenFileNode *fileNodePtr;
    PciCanChanData *chd;
    unsigned long timeLeft;
    unsigned long timeout;
    wait_queue_t wait;
    int ret;
    
    fileNodePtr = filp->private_data;
    chd = fileNodePtr->chanData;
    
    switch (ioctl_cmd) {
        /*------------------------------------------------------------------*/
    case VCAN_IOC_SENDMSG:
        
        if (!fileNodePtr->writeIsBlock && txQFull(chd)) return -EAGAIN;
        
        init_waitqueue_entry(&wait, current);
        add_wait_queue(&chd->txChanWaitQ, &wait);
        
        while(1) {
            current->state = TASK_INTERRUPTIBLE;
            spin_lock(&chd->sendQLock);
            if (txQFull(chd)) {
                spin_unlock(&chd->sendQLock);
                if (fileNodePtr->writeTimeout != -1){
                    if (schedule_timeout(1 + fileNodePtr->writeTimeout * HZ/1000) == 0) {
                        /* Transmit timed out */
                        remove_wait_queue(&chd->txChanWaitQ, &wait);		    
                        return -EAGAIN;
                    }
                    
                } else {
                    schedule();
                }
                
                if (signal_pending(current)) {
                    /* Sleep was interrupted by signal */
                    remove_wait_queue(&chd->txChanWaitQ, &wait);		    
                    return -ERESTARTSYS;
                }
                
            } else {
                CAN_MSG *bufMsgPtr = &(chd->txChanBuffer[chd->txChanBufHead]);
                current->state = TASK_RUNNING;
                remove_wait_queue(&chd->txChanWaitQ, &wait);		    
                
                if (copy_from_user(bufMsgPtr, (CAN_MSG*)arg, sizeof(CAN_MSG))) {
                    spin_unlock(&chd->sendQLock);		    
                    return -EFAULT;
                }
                /* This is for keeping track of the originating fileNode */
                bufMsgPtr->user_data = fileNodePtr->transId;
                bufMsgPtr->flags &= ~(VCAN_MSG_FLAG_TX_NOTIFY | VCAN_MSG_FLAG_TX_START);
                if (fileNodePtr->modeTx || (chd->fileOpenCount > 1)) bufMsgPtr->flags |= VCAN_MSG_FLAG_TX_NOTIFY;
                if (fileNodePtr->modeTxRq) bufMsgPtr->flags |= VCAN_MSG_FLAG_TX_START;
                
                if (chd->txChanBufHead >= (TX_CHAN_BUF_SIZE - 1)) chd->txChanBufHead = 0;
                else chd->txChanBufHead++;
                spin_unlock(&chd->sendQLock);
                /* Exit loop */
                break;
            }
        }
        
        /* Add pcicanSend to the immediate task queue */
        if (pciCanTxAvailable(chd)){ 
            queue_task(&chd->txTaskQ, &tq_immediate);
            mark_bh(IMMEDIATE_BH);
        }
        
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_RECVMSG:
        
        if (!fileNodePtr->readIsBlock && rxQEmpty(fileNodePtr)) return -EAGAIN;
        
        init_waitqueue_entry(&wait, current);
        add_wait_queue(&fileNodePtr->rxWaitQ, &wait);
        
        while(1) {
            current->state = TASK_INTERRUPTIBLE;
            
            if (rxQEmpty(fileNodePtr)) {
                
                if (fileNodePtr->readTimeout != -1){
                    if (schedule_timeout(1 + fileNodePtr->readTimeout * HZ/1000) == 0) {
                        /* Receive timed out */
                        remove_wait_queue(&fileNodePtr->rxWaitQ, &wait);
                        return -EAGAIN;
                    }
                } else {
                    schedule();
                }
                
                if (signal_pending(current)) {
                    /* Sleep was interrupted by signal */
                    remove_wait_queue(&fileNodePtr->rxWaitQ, &wait);
                    return -ERESTARTSYS;
                }
                
            } else {
                current->state = TASK_RUNNING;
                remove_wait_queue(&fileNodePtr->rxWaitQ, &wait);		    
                
                copy_to_user_ret((VCAN_EVENT*) arg, 
                    &(fileNodePtr->fileRcvBuffer [fileNodePtr->rcvBufTail]), 
                    sizeof(VCAN_EVENT), 
                    -EFAULT);
                if (++(fileNodePtr->rcvBufTail) >= FILE_RCV_BUF_SIZE)
                    fileNodePtr->rcvBufTail = 0;
                /* Exit loop */
                break;
            }
        }
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_BUS_ON:
        pciCanBusOn(chd);
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_BUS_OFF:
        pciCanBusOff(chd);
        vCanFlushSendBuffer(chd);
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_SET_BITRATE:
        {
            VCanBusParams busParams;
            copy_from_user_ret(&busParams, (VCanBusParams*)arg, sizeof(VCanBusParams), -EFAULT);
            if (pciCanSetBusParams (chd, &busParams)) {
                busParams.freq = 0;
            } else {
                pciCanGetBusParams (chd, &busParams);
            }
            copy_to_user_ret((VCanBusParams*)arg, &busParams, sizeof(VCanBusParams), -EFAULT);
        }
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_GET_BITRATE:
        {
            VCanBusParams busParams;
            pciCanGetBusParams (chd, &busParams);        
            copy_to_user_ret((VCanBusParams*)arg, &busParams, sizeof(VCanBusParams), -EFAULT);
        }
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_SET_OUTPUT_MODE:
        pciCanSetOutputMode(chd, arg);
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_SET_MSG_FILTER:
        copy_from_user_ret(&(fileNodePtr->filter), (VCanMsgFilter*) arg, sizeof(VCanMsgFilter), -EFAULT);
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_GET_MSG_FILTER:
        copy_to_user_ret((VCanMsgFilter*) arg, &(fileNodePtr->filter), sizeof(VCanMsgFilter), -EFAULT);
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_OPEN_CHAN:
        spin_lock(&chd->openLock );
        {
            PciCanOpenFileNode *tmpFnp;
            for (tmpFnp = chd->openFileList; tmpFnp && !tmpFnp->channelLocked; tmpFnp = tmpFnp->next);
            
            if (tmpFnp) {
                ret = put_user(-1, (int*) arg);		    
            } else {
                fileNodePtr->channelOpen = 1;
                ret = put_user(0, (int*) arg);		    
            }
        }
        spin_unlock(&chd->openLock );
        if (ret) return -EFAULT;
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_OPEN_EXCL:
        spin_lock(&chd->openLock );
        {
            PciCanOpenFileNode *tmpFnp;
            for (tmpFnp = chd->openFileList; tmpFnp && !tmpFnp->channelOpen; tmpFnp = tmpFnp->next);
            
            if (tmpFnp) {
                ret = put_user(-1, (int*) arg);		    
            } else {
                fileNodePtr->channelOpen = 1;
                fileNodePtr->channelLocked = 1;
                ret = put_user(0, (int*) arg);		    
            }
        }
        spin_unlock(&chd->openLock );
        if (ret) return -EFAULT;
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_WAIT_EMPTY:
        get_user_ret(timeout, (unsigned long*) arg, -EFAULT);
        
        init_waitqueue_entry(&wait, current);
        add_wait_queue(&chd->flushQ, &wait);
        current->state = TASK_INTERRUPTIBLE;
        timeLeft = -1;
        
        if ((getQLen(chd->txChanBufHead, chd->txChanBufTail, TX_CHAN_BUF_SIZE) != 0) ||
            ((chd->sjaState.state != CHIPSTAT_BUSOFF) && !pciCanTxAvailable(chd))) {		
            /* Sleep for [timeout] milliseconds */
            timeLeft = schedule_timeout(1 + timeout * HZ / 1000);				
        }      
        
        remove_wait_queue(&chd->flushQ, &wait);
        current->state = TASK_RUNNING;
        if (timeLeft == 0) return -EAGAIN;
        if (signal_pending(current)) return -ERESTARTSYS;
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_FLUSH_RCVBUFFER:
        fileNodePtr->rcvBufHead = 0;
        fileNodePtr->rcvBufTail = 0;
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_FLUSH_SENDBUFFER:
        vCanFlushSendBuffer(chd);
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_SET_WRITE_BLOCK:
        fileNodePtr->writeIsBlock = arg;
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_SET_READ_BLOCK:
        fileNodePtr->readIsBlock = arg;
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_SET_WRITE_TIMEOUT:
        fileNodePtr->writeTimeout = arg;
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_SET_READ_TIMEOUT:
        fileNodePtr->readTimeout = arg;
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_READ_TIMER:
        put_user_ret(pciCanTime(chd->usPerTick), (int*) arg, -EFAULT);
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_GET_TX_ERR:
        put_user_ret(inb(chd->sja1000 + PCAN_TXERR), (int*) arg, -EFAULT);	
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_GET_RX_ERR:
        put_user_ret(inb(chd->sja1000 + PCAN_RXERR), (int*) arg, -EFAULT);	
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_GET_OVER_ERR:
        put_user_ret(fileNodePtr->overruns, (int*) arg, -EFAULT);	
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_GET_RX_QUEUE_LEVEL:
        {
            int ql = getQLen(fileNodePtr->rcvBufHead, fileNodePtr->rcvBufTail, FILE_RCV_BUF_SIZE);
            put_user_ret(ql, (int*) arg, -EFAULT);	
        }
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_GET_TX_QUEUE_LEVEL:
        {
            int ql = getQLen(chd->txChanBufHead, chd->txChanBufTail, TX_CHAN_BUF_SIZE);
            put_user_ret(ql, (int*) arg, -EFAULT);	
        }
        break;
        /*------------------------------------------------------------------*/
    case VCAN_IOC_GET_CHIP_STATE:
        {
            pciCanRequestChipState(chd);
            put_user_ret(chd->sjaState.state, (int*) arg, -EFAULT);	
        }
        break;
        /*------------------------------------------------------------------*/
        
    default:
        return -EINVAL;
    }
    /***************************************************************************/
    
    return 0;
}

/*======================================================================*/
/*  Process send Q - This function is called from the immediate queue   */
/*======================================================================*/

static void pcicanSend (void *void_chanData)
{
    PciCanChanData *chd = (PciCanChanData*) void_chanData;
    
    if (!pciCanTxAvailable(chd)) return;
    
    /* Send Messages */
    if (getQLen(chd->txChanBufHead, chd->txChanBufTail, TX_CHAN_BUF_SIZE) != 0) {
        
        pciCanTransmitMessage(chd, &(chd->txChanBuffer[chd->txChanBufTail]));
        /* ??? Multiple CPU race when flushing txQ */
        if (++(chd->txChanBufTail) >= TX_CHAN_BUF_SIZE) chd->txChanBufTail  = 0;
        wake_up_interruptible(&chd->txChanWaitQ );
    } else {
        wake_up_interruptible(&chd->flushQ);
    }
    
    return;
}

/*======================================================================*/
/*  Check sja1000 health                                                */
/*======================================================================*/

static int pciCanProbeChannel (PciCanChanData *chan)
{
    unsigned int port, tmp;
    unsigned addr;
    
    addr = chan->sja1000;
    
    // First, reset the chip.
    outb (0x01, addr + PCAN_MOD);
    port = inb (addr + PCAN_MOD);
    
    // If we don't read 0x01 back, then it isn't an sja1000.
    if ((port & 0x01) == 0) return -1;
    
    // 0xff is not a valid answer.
    if (port == 0xFF) return -1;
    
    // Try to set the Pelican bit.
    port = inb (addr + PCAN_CDR);
    outb ((unsigned char) (port | PCAN_PELICAN), addr + PCAN_CDR);
    port = inb (addr + PCAN_CDR);
    if ((port & PCAN_PELICAN) == 0) return -1;
    
    // Reset it..
    outb ((unsigned char) (port & ~PCAN_PELICAN), addr + PCAN_CDR);
    port = inb(addr + PCAN_CDR);
    if ((port & PCAN_PELICAN) != 0) return -1;
    
    // Check that bit 5 in the 82c200 control register is always 1
    tmp = inb (addr + 0);
    if ((tmp & 0x20) == 0) return -1;
    outb ( (unsigned char) (tmp | 0x20), addr + 0);
    tmp = inb (addr + 0);
    if ((tmp & 0x20) == 0) return -1;
    
    // The 82c200 command register is always read as 0xFF
    tmp = inb (addr + 1);
    if (tmp != 0xFF) return -1;
    outb ( 0, addr + 1);
    tmp = inb (addr + 1);
    if (tmp != 0xFF) return -1;
    
    // Set the Pelican bit.
    port = inb (addr + PCAN_CDR);
    outb ( (unsigned char) (port | PCAN_PELICAN), addr + PCAN_CDR);
    port = inb (addr + PCAN_CDR);
    if ((port & PCAN_PELICAN) == 0) return -1;
    return 0;
} /* pciCanProbeChannel */

/*======================================================================*/
/* Find out some info about the H/W                                     */
/* (*cd) must have pciIf, xilinx and sjaBase initialized                */
/*======================================================================*/

int pciCanProbe (PciCanCardData *cd)
{
    int i;
    unsigned addr;
    int xilinxRev;
    
    // Set (one) Wait State needed(?) by the CAN chip on the ADDON bus of S5920
    // WritePortUlong(ci->cc.s5920_address + S5920_PTCR, 0x81818181L );
    
    // Assert PTADR# - we're in passive mode so the other bits are not important
    outl (0x80808080L, cd->pciIf + S5920_PTCR);
    
    xilinxRev = inb(cd->xilinx + XILINX_VERINT) >> 4;
    // ci->cc.sw_revision = xilinx_rev;
    cd->firmwareVersionMajor = xilinxRev;
    cd->firmwareVersionMinor = 0;
    cd->firmwareVersionBuild = 0;
    
    cd->cardEeprom.address_out = cd->xilinx + XILINX_OUTA;
    cd->cardEeprom.address_in  = cd->xilinx + XILINX_INA;
    cd->cardEeprom.in_mask = 0x80;
    cd->cardEeprom.out_mask = 0x80;
    
    /* Check for piggybacks and other data stored in the Dallas memories. */
    
    ds_init (&(cd->cardEeprom));
    
    if (ds_check_for_presence_loop (&(cd->cardEeprom))) {
        unsigned char buf[32];
        unsigned long boardSerialNumber;
        
        ds_read_memory(&(cd->cardEeprom), 0, buf, sizeof(buf));
        memcpy(&boardSerialNumber, &buf[6], sizeof(boardSerialNumber));
        
        if (boardSerialNumber != 0xffffffff) {
            cd->serialNumber = boardSerialNumber;
        }
#if 0      
        memset(cd->ean, 0, sizeof(cd->ean));
        if (memcmp(&buf[1], "\xff\xff\xff\xff\xff\xff\xff\xff", 6) != 0) {
            packed_EAN_to_BCD_with_csum(&buf[1], cd->ean);
        }
#endif      
#if 0
        {
            unsigned int sum = 0, n;
            
            DbgPrint ("EEPROM Board: ");
            for (n = 0; n < 32; ++n) {
                DbgPrint ("%02x ", buf [n]);
                sum += buf [n];
            }
            DbgPrint (", sum=%02x\n", sum);
            
            DbgPrint ("serial_number = 0x%04x (%u)\n",
                ci->serial_number,
                ci->serial_number);
            
            DbgPrint ("ean_code = ");
            for (n=0; n < sizeof(ci->ean_code); n++) {
                DbgPrint("%02x", ci->ean_code[n]);
            }
            DbgPrint ("\n");
        }
#endif
    }
    
    for (i = 0; i < MAX_CHANNELS; i++) {
        PciCanChanData *chd = &cd->chanData[i];
        
        if (chd->chipType == CAN_CHIP_TYPE_UNKNOWN) {
        /* This does not work, if the card is probed the second time (e.g. after
	       wakeup from sleep mode), and there are less than MAX_CHANNELS available.
           qqq
            */
            // Each controller has PCICAN_BYTES_PER_CIRCUIT bytes.
            // This is "hardcoded" on the PCB and in the Xilinx.
            addr = cd->sjaBase + (i * PCICAN_BYTES_PER_CIRCUIT);
            chd->channel = i;
            chd->sja1000 = addr;
            
            chd->chanEeprom.address_out = 0;
            chd->chanEeprom.address_in  = 0;
            chd->chanEeprom.in_mask = 0;
            chd->chanEeprom.out_mask = 0;
            
            chd->xilinxAddressOut = 0;
            chd->xilinxAddressCtrl= 0;
            chd->xilinxAddressIn  = 0;
            
            if (chd->channel == 0) {
                chd->chanEeprom.address_out = cd->xilinx + XILINX_OUTA;
                chd->chanEeprom.address_in  = cd->xilinx + XILINX_INA;
                chd->chanEeprom.in_mask = 0x01;
                chd->chanEeprom.out_mask = 0x01;
                
                chd->xilinxAddressOut = cd->xilinx + XILINX_OUTA;
                chd->xilinxAddressCtrl= cd->xilinx + XILINX_CTRLA;
                chd->xilinxAddressIn  = cd->xilinx + XILINX_INA;
            }
            else if (chd->channel == 1) {
                chd->chanEeprom.address_out = cd->xilinx + XILINX_OUTB;
                chd->chanEeprom.address_in  = cd->xilinx + XILINX_INB;
                chd->chanEeprom.in_mask = 0x01;
                chd->chanEeprom.out_mask = 0x01;
                
                chd->xilinxAddressOut = cd->xilinx + XILINX_OUTB;
                chd->xilinxAddressCtrl= cd->xilinx + XILINX_CTRLB;
                chd->xilinxAddressIn  = cd->xilinx + XILINX_INB;
            }
        }
        else {
            DEBUG(1, "Channel %d already detected - don't calculate address\n", chd->channel);
        }
        
        if (pciCanProbeChannel (chd) == 0) {
            chd->chipType = CAN_CHIP_TYPE_SJA1000;
        } else {
            /* Exit from loop */
            break;
        }
    }
    
    cd->nrChannels = i;
    DEBUG(1, "Kvaser PCIcan with %d channels found\n", cd->nrChannels);
    
    //
    // Init Dallas ports so we can read the memories on the piggybacks, if
    // they are present.
    //
    
    outb(1, cd->xilinx + XILINX_CTRLA);
    outb(1, cd->xilinx + XILINX_CTRLB);
    
    for (i = 0; i < cd->nrChannels; ++i) {
        PciCanChanData *chd = &cd->chanData[i];
        
        chd->transType = VCAN_TRANSCEIVER_TYPE_251;
        
        ds_init(&chd->chanEeprom);
        
        memset(chd->ean, 0, sizeof(chd->ean));
        chd->serialLow = 0;
        chd->serialHigh = 0;
        
        if (ds_check_for_presence_loop (&chd->chanEeprom)) {
            
            unsigned char buf[32];
            
            ds_read_memory (&chd->chanEeprom, 0, buf, sizeof(buf));
            
            if ((buf[9] == VCAN_TRANSCEIVER_TYPE_SWC) ||
                (buf[9] == VCAN_TRANSCEIVER_TYPE_251) ||
                (buf[9] == VCAN_TRANSCEIVER_TYPE_GAL) ||
                (buf[9] == VCAN_TRANSCEIVER_TYPE_252)) {
                chd->transType = buf[9];
            }
            
            /* qqq The following fields should probably also be filled in:
            unsigned int  transceiver_type;
            unsigned int  transceiver_state;
            unsigned long serial_number_low,
            serial_number_high;
            unsigned long transceiver_capabilities;
            char          ean[16];
            unsigned int  linemode,
            resnet;
            */
            memset(chd->ean, 0, sizeof(chd->ean));
#if 0
            packed_EAN_to_BCD_with_csum(&buf[0], chd->ean);
#endif
#if 0
            if (DBGF(DEBUG_INIT)) {
                unsigned int sum = 0, n;
                DbgPrint ("EEPROM Channel %d: ", chd->channel);
                for (n = 0; n < 32; ++n) {
                    DbgPrint ("%02x ", buf [n]);
                    sum += buf [n];
                }
                DbgPrint (", sum=%02x\n", sum);
                DbgPrint ("%s: channel %d piggy %02x (%d)\n",
                    hwif.hw_name, chd->channel,
                    chd->transceiver_info.transceiver_type,
                    chd->transceiver_info.transceiver_type);
            }
#endif
        }
        
        switch (chd->transType) {
            
        case VCAN_TRANSCEIVER_TYPE_SWC:
            chd->lineMode = VCAN_TRANSCEIVER_LINEMODE_SWC_NORMAL;
            break;
            
        case VCAN_TRANSCEIVER_TYPE_251:
        case VCAN_TRANSCEIVER_TYPE_NONE:
            chd->lineMode = VCAN_TRANSCEIVER_LINEMODE_NORMAL;
            break;
            
        default:
            chd->lineMode = VCAN_TRANSCEIVER_LINEMODE_NA;
            break;
        }
    }
    
    //
    // Now restore the control registers in the Xilinx to the default config;
    // needed for talking to a PCIcan-Q. If we have piggys present, they will
    // be setup again in hermes_setup_transceiver().
    //
    
    if (cd->nrChannels <= 2) {
        // PCIcan-S, -D. Disable the ununsed interrupts so Galathea will work.
        // This is done by writing 0x40 to the control ports.
        // Also, for Galathea, set bit 1 to 1 in the control registers.
        outb(0x42, cd->xilinx + XILINX_CTRLA);
        outb(0x42, cd->xilinx + XILINX_CTRLB);
    } else {
        // Xilinx setup for PCIcan-Q.
        outb(0, cd->xilinx + XILINX_CTRLA);
        outb(0, cd->xilinx + XILINX_CTRLB);
    }
    
#if 0
    // Useful for debug.
    { int i, j;
    
    unsigned  x = cd->sjaBase;
    
    for (i=0; i<64; i++) {
        printk("<1>%04x  ", x + i*16);
        for (j=0; j<16; j++) {
            printk("%02x ", inb(x + i*16 + j));
        }
        printk("\n");
    }
    }
#endif
    
    if (i == 0) {
        /* no channels found */
        cd->cardPresent = 0;
        return -1;
    }
    
    cd->cardPresent = 1;
    
    return 0;
} /* pciCanProbe */


/*======================================================================*/
/* Perform transceiver-specific setup on PCIcan with piggybacks         */
/*======================================================================*/

static void pciCanSetupTransceiver (PciCanChanData *chd)
{
    if (chd->xilinxAddressCtrl == 0) return;
    
    switch (chd->transType) {
        
    case VCAN_TRANSCEIVER_TYPE_NONE:
    case VCAN_TRANSCEIVER_TYPE_251:
        /* Piggyback pins all inputs. */
        outb (0, chd->xilinxAddressCtrl);
        break;
        
    case VCAN_TRANSCEIVER_TYPE_SWC:
    /* Pin 1 = HVOLT#, input
	   Pin 2 = POW#, input
       Pin 3 = NSTB, output
       Pin 4 = EN, output
        */
        outb (0x18, chd->xilinxAddressCtrl);
        // qqq we should implement a "sticky" input register in the Xilinx
        // qqq so we can see if HVOLT has gone low.
        break;
        
        //
        // The 252/1053 code is not yet tested. qqq
        //
    case VCAN_TRANSCEIVER_TYPE_252:
    /* Pin 1 = NERR#, input
	   Pin 2 = STB#, output
       Pin 3 = EN, output
       Pin 4 = WAK#, output
        */
        outb (0x0e, chd->xilinxAddressCtrl);
        // qqq we should implement a "sticky" input register in the Xilinx
        // qqq so we can see if NERR has gone low.
        break;
    case VCAN_TRANSCEIVER_TYPE_GAL:
        {
            // Activate the transceiver. OUTx register bit 1 high then low
            
            unsigned char tmp;
            tmp = inb(chd->xilinxAddressOut);
            outb((unsigned char)(tmp | 2), chd->xilinxAddressOut);
            outb((unsigned char)(tmp & ~2), chd->xilinxAddressOut);
            
        }
        break;
    default:
        // qqq unsupported piggy
        printk ("<1> PCIcan: unknown piggy on chan %d\n", chd->channel);
        break;
    }
} /* pciCanSetupTransceiver */


/*======================================================================*/
/* This sets the transceiver to the specified line mode.                */
/* It's a no-op for 251-type tranceivers, but for e.g. SWCs the current */
/* line mode could be WAKEUP.                                           */
/*======================================================================*/

static void pciCanActivateTransceiver (PciCanChanData *chd, int linemode)
{
    if (chd->xilinxAddressOut == 0) return;
    
    switch (chd->transType) {
        
    case VCAN_TRANSCEIVER_TYPE_SWC:
        
    /* Pin 1 = HVOLT#, input
	   Pin 2 = POW#, input
       Pin 3 = NSTB, output
       Pin 4 = EN, output
        */
        switch (linemode) {
            
        case VCAN_TRANSCEIVER_LINEMODE_SWC_SLEEP:
            outb (0x00, chd->xilinxAddressOut);
            break;
            
        case VCAN_TRANSCEIVER_LINEMODE_SWC_NORMAL:
            outb (0x18, chd->xilinxAddressOut);
            break;
            
        case VCAN_TRANSCEIVER_LINEMODE_SWC_FAST:
            outb (0x08, chd->xilinxAddressOut);
            break;
            
        case VCAN_TRANSCEIVER_LINEMODE_SWC_WAKEUP:
            outb (0x10, chd->xilinxAddressOut);
            break;
        }
        break;
        
        case VCAN_TRANSCEIVER_TYPE_252:
            /* EN=1, STB#=1, WAK#=1 */
            outb ( 0x0e, chd->xilinxAddressOut);
            break;
        case VCAN_TRANSCEIVER_TYPE_GAL:
            
            break;
        default:
            break;
    }
} /* pciCanActivateTransceiver */


/*======================================================================*/
/* Enable bus error interrupts, and reset the                           */
/* counters which keep track of the error rate                          */
/*======================================================================*/

static void pciCanResetErrorCounter (PciCanChanData *chd)
{
    unsigned char ier;
    ier = inb(chd->sja1000 + PCAN_IER);
    outb(ier | PCAN_BEIE, chd->sja1000 + PCAN_IER);
    chd->errorCount = 0;
    
    chd->errorTime = pciCanTime(chd->usPerTick);
} /* pciCanResetErrorCounter */


/*======================================================================*/
/*  Set bit timing                                                      */
/*======================================================================*/

int pciCanSetBusParams (PciCanChanData *chd, VCanBusParams *par)
{
    unsigned quantaPerCycle;
    unsigned long brp;
    unsigned char cbt0;
    unsigned char cbt1;
    unsigned char tmp;
    int resetStatus;
    unsigned long freq;
    unsigned char tseg1, tseg2, sjw, sam;
    unsigned circAddr = chd->sja1000;
    
    freq  = par->freq;
    sjw   = par->sjw;
    tseg1 = par->tseg1;
    tseg2 = par->tseg2;
    sam   = par->samp3;
    sjw--;
    
    quantaPerCycle = tseg1 + tseg2 + 1;
    if (quantaPerCycle == 0 || freq == 0) return -1;
    
    brp = (8000000L * 64) / (freq * quantaPerCycle);
    if ((brp & 0x3F) != 0) {
        // Fraction != 0 : not divisible.
        return -1;
    }
    brp = (brp >> 6) - 1;
    if (brp > 64 || sjw > 3 || quantaPerCycle < 3) {
        return -1;
    }
    
    cbt0 = (sjw << 6) + brp;
    cbt1 = ((sam==3?1:0)<<7) + ((tseg2-1)<<4) + (tseg1-1);
    
    /* Put the circuit in Reset Mode */
    tmp = inb(circAddr + PCAN_MOD);
    
    // Always set the AFM bit.
    tmp |= PCAN_AFM;
    resetStatus = tmp & PCAN_RM;
    outb( tmp | PCAN_RM | PCAN_AFM, circAddr + PCAN_MOD);
    
    outb(cbt0, circAddr + PCAN_BTR0);
    outb(cbt1, circAddr + PCAN_BTR1);
    
    if (!resetStatus) {
        tmp = inb(circAddr + PCAN_MOD);
        outb (tmp & ~PCAN_RM, circAddr + PCAN_MOD);
    }
    
    return 0;
} /* pciCanSetBusParams */

/*======================================================================*/
/*  Get bit timing                                                      */
/*======================================================================*/

static int pciCanGetBusParams (PciCanChanData *chd, VCanBusParams *par)
{
    unsigned quantaPerCycle;
    unsigned char cbt0;
    unsigned char cbt1;
    unsigned long brp;
    
    cbt0 = inb(chd->sja1000 + PCAN_BTR0);
    cbt1 = inb(chd->sja1000 + PCAN_BTR1);
    
    par->sjw = 1 + (cbt0 >> 6);
    par->samp3 = (cbt1 >> 7) == 1 ? 3 : 1;
    
    par->tseg1 = 1 + (cbt1 & 0xf);
    par->tseg2 = 1 + ((cbt1 >> 4) & 0x7);
    
    quantaPerCycle = par->tseg1 + par->tseg2 + 1;    
    brp = 1 +(cbt0 & 0x3f);
    
    par->freq = 8000000L/(quantaPerCycle * brp);
    
    return 0;
} /* pciCanGetBusParams */


/*======================================================================*/
/*  Set silent or normal mode                                           */
/*======================================================================*/

int pciCanSetOutputMode (PciCanChanData *chd, int silent)
{
    unsigned char driver;
    unsigned char tmp;
    
    // Save control register
    tmp = inb (chd->sja1000 + PCAN_MOD);
    // Put the circuit in Reset Mode
    outb (tmp | PCAN_RM , chd->sja1000 + PCAN_MOD);
    // Always set the AFM bit.
    tmp |= PCAN_AFM;
    
    if (chd->transType == VCAN_TRANSCEIVER_TYPE_GAL) {
        driver = OCR_DEFAULT_GAL;
    } else {
        driver = OCR_DEFAULT_STD;
    }
    
    if(silent) {
        tmp |= PCAN_LOM;
    } else {
        tmp &= ~PCAN_LOM;
    }
    
    // Set the output control
    outb (driver, chd->sja1000 + PCAN_OCR);
    // Restore control register
    outb (tmp, chd->sja1000 + PCAN_MOD);
    
    return 0;
} /* pciCanSetOutputMode */

/*======================================================================*/
/*  Line mode                                                           */
/*======================================================================*/

int pciCanSetTranceiverMode (PciCanChanData *chd, int linemode, int resnet)
{
    chd->lineMode = linemode;
    pciCanActivateTransceiver (chd, chd->lineMode);
    return 0;
} /* pciCanSetTranceiverMode */

/*======================================================================*/
/*  Query chip status                                                   */
/*======================================================================*/

int pciCanRequestChipState (PciCanChanData *chd)

{
    VCAN_EVENT msg;
    
    unsigned char sr, cr;
    
    sr = inb (chd->sja1000 + PCAN_SR);
    cr = inb (chd->sja1000 + PCAN_MOD);
    
    chd->sjaState.txerr = inb(chd->sja1000 + PCAN_TXERR);
    chd->sjaState.rxerr = inb(chd->sja1000 + PCAN_RXERR);
    
    switch (sr & (PCAN_BS|PCAN_ES)) {
    case PCAN_BS:
        chd->sjaState.state = CHIPSTAT_BUSOFF;
        break;
        
    case PCAN_BS|PCAN_ES:
        chd->sjaState.state = CHIPSTAT_BUSOFF;
        break;
        
    case PCAN_ES:
        chd->sjaState.state = CHIPSTAT_ERROR_WARNING;
        if ((chd->sjaState.txerr > 127) ||
            (chd->sjaState.rxerr > 127)) {
            chd->sjaState.state |= CHIPSTAT_ERROR_PASSIVE;
        }
        break;
        
    case 0:
        chd->sjaState.state = CHIPSTAT_ERROR_ACTIVE;
        break;
    }
    
    if (cr & PCAN_RM) {
        // It's in reset mode. We report BUSOFF but should really be
        // reporting "inactive" or so. qqq
        chd->sjaState.state = CHIPSTAT_BUSOFF;
    }
    
    msg.tag = V_CHIP_STATE;
    msg.timeStamp = pciCanTime(chd->usPerTick);
    msg.transId = 0;
    msg.tagData.chipState.busStatus = (unsigned char) chd->sjaState.state;
    msg.tagData.chipState.txErrorCounter = (unsigned char) chd->sjaState.txerr;
    msg.tagData.chipState.rxErrorCounter = (unsigned char) chd->sjaState.rxerr;
    pciCanDispatchEvent(chd, &msg);
    
    return 0;
} /* pciCanRequestChipState */


/*======================================================================*/
/*  Go bus on                                                           */
/*======================================================================*/

int pciCanBusOn (PciCanChanData *chd)
{
    unsigned tmp;
    
    switch (chd->transType) {
        
    case VCAN_TRANSCEIVER_TYPE_SWC:
        chd->lineMode = VCAN_TRANSCEIVER_LINEMODE_SWC_NORMAL;
        break;
        
    case VCAN_TRANSCEIVER_TYPE_251:
    case VCAN_TRANSCEIVER_TYPE_NONE:
        chd->lineMode = VCAN_TRANSCEIVER_LINEMODE_NORMAL;
        break;
        
    default:
        chd->lineMode = VCAN_TRANSCEIVER_LINEMODE_NA;
        break;
    }
    
    pciCanSetupTransceiver(chd);
    pciCanActivateTransceiver(chd, chd->lineMode);
    
    chd->isOnBus = 1;
    
    chd->overrun = 0;
    pciCanResetErrorCounter(chd);
    
    /* Go on bus */
    tmp = inb (chd->sja1000 + PCAN_MOD);
    // Always set the AFM bit.
    tmp |= PCAN_AFM;
    outb(PCAN_CDO, chd->sja1000 + PCAN_CMR);
    outb(tmp | PCAN_RM, chd->sja1000 + PCAN_MOD);
    outb(0, chd->sja1000 + PCAN_TXERR);
    outb(0, chd->sja1000 + PCAN_RXERR);
    (void)inb (chd->sja1000 + PCAN_ECC);
    
    // Write the hardware filters once again. They might have been corrupted
    // if we tried to write a message to the transmit buffer at the same
    // time as the sja1000 decided to go bus off due to e.g. excessive errors.
    outb(0, chd->sja1000 + PCAN_ACR0);
    outb(0, chd->sja1000 + PCAN_ACR1);
    outb(0, chd->sja1000 + PCAN_ACR2);
    outb(0, chd->sja1000 + PCAN_ACR3);
    outb(0xFF, chd->sja1000 + PCAN_AMR0);
    outb(0xFF, chd->sja1000 + PCAN_AMR1);
    outb(0xFF, chd->sja1000 + PCAN_AMR2);
    outb(0xFF, chd->sja1000 + PCAN_AMR3);
    
    outb(tmp & ~PCAN_RM, chd->sja1000 + PCAN_MOD);
    
    chd->chipState = CHIPSTAT_ERROR_ACTIVE;
    pciCanRequestChipState(chd);
    return 0;
} /* pciCanBusOn */

/*======================================================================*/
/*  Go bus off                                                          */
/*======================================================================*/

int pciCanBusOff (PciCanChanData *chd)
{
    unsigned tmp;
    
    chd->isOnBus = 0;
    
    tmp = inb(chd->sja1000 + PCAN_MOD);
    outb(tmp | PCAN_RM, chd->sja1000 + PCAN_MOD);
    
    pciCanRequestChipState(chd);
    
    return 0;
} /* pciCanBusOff */


/*======================================================================*/
/*  Disable Card                                                        */
/*======================================================================*/
int pciCanResetCard (PciCanCardData *cd)
{
    unsigned tmp;
    
    /* The card must be present! */
    if (!cd->cardPresent) return -1;
    
    /* Disable interrupts from card */
    tmp = inl(cd->pciIf + S5920_INTCSR);
    tmp &= ~INTCSR_ADDON_INTENABLE_M;
    outl(tmp, cd->pciIf + S5920_INTCSR);
    
    return 0;
} /* pciCanResetCard */

/*======================================================================*/
/*  Enable Card                                                         */
/*======================================================================*/

int pciCanInitHW  (PciCanCardData *cd)
{
    unsigned tmp;
    
    /* The card must be present! */
    if (!cd->cardPresent) return -1;
    
    /* enable interrupts from card */    
    tmp = inl (cd->pciIf + S5920_INTCSR);
    tmp |= INTCSR_ADDON_INTENABLE_M;
    outl(tmp, cd->pciIf + S5920_INTCSR);
    
    return 0;
} /* hermes_setup_hw */



/*======================================================================*/
/*  Interrupt handling functions                                        */
/*======================================================================*/

static void pciCanReceiveIsr (PciCanChanData *chd)
{
    unsigned circAddr = chd->sja1000;
    static VCAN_EVENT e;
    int i, r;
    unsigned char dlc, flags;
    unsigned char* p;
    unsigned data;
    unsigned char SR;
    WL id;

    SR = inb(circAddr + PCAN_SR);
    
    while (SR & PCAN_RBS) {
        
        unsigned char tmp;
        
        e.timeStamp = pciCanTime(chd->usPerTick);
        
        tmp = inb(circAddr + PCAN_MSGBUF);
        dlc = (unsigned char)(tmp & 0x0F);
        if (dlc > 8) dlc = 8;
        flags = (unsigned char)((tmp & PCAN_FF_REMOTE) ? VCAN_MSG_FLAG_REMOTE_FRAME : 0);
        id.L = 0;
        
        if (tmp & PCAN_FF_EXTENDED) { /* Extended CAN */
            id.B.b3 = inb (circAddr + PCAN_XID0);
            id.B.b2 = inb (circAddr + PCAN_XID1);
            id.B.b1 = inb (circAddr + PCAN_XID2);
            id.B.b0 = inb (circAddr + PCAN_XID3);
            id.L >>= 3;
            id.L |= VCAN_EXT_MSG_ID;
            data = circAddr + PCAN_XDATA;
        }
        else { /* Standard CAN */
            id.B.b1 = inb (circAddr + PCAN_SID0);
            id.B.b0 = inb (circAddr + PCAN_SID1);
            id.L >>= 5;
            data = circAddr + PCAN_SDATA;
        }

        p = e.tagData.msg.data;
        
        for (i=0; i<dlc; i++) {
            *p++ = inb (data++);
        }

        if (chd->overrun) {
            flags |= chd->overrun;
            chd->overrun = 0;
        }
        
        e.tag = V_RECEIVE_MSG;
        e.transId = 0;
        e.tagData.msg.id = id.L;
        e.tagData.msg.flags = flags;
        e.tagData.msg.dlc = dlc;
        
        //    printk("<1>in isr: id=%d\n", e.tagData.msg.id)
        r = pciCanDispatchEvent(chd, &e);
        // Release receive buffer
        
        outb(PCAN_RRB, circAddr + PCAN_CMR);
        
        SR = inb (circAddr + PCAN_SR);
    }
    
    if (chd->errorCount > 0) pciCanResetErrorCounter(chd);
} /* pciCanReceiveIsr */

/*======================================================================*/
/*  Transmit interrupt handler                                          */
/*======================================================================*/

static void pciCanTransmitIsr (PciCanChanData *chd)
{
    
    pciCanActivateTransceiver(chd, chd->lineMode);
    
    /* "send" a transmit ack. */
    if (chd->currentTxMsg->flags & VCAN_MSG_FLAG_TXACK) {
        VCAN_EVENT *e = (VCAN_EVENT*) chd->currentTxMsg;
        e->tag = V_RECEIVE_MSG;
        e->timeStamp = pciCanTime(chd->usPerTick);
        e->tagData.msg.flags &= ~VCAN_MSG_FLAG_TXRQ;
        
        pciCanDispatchEvent(chd, e);
    }
    
    /* Send next message in queue */
    queue_task(&chd->txTaskQ, &tq_immediate);
    mark_bh(IMMEDIATE_BH);
    
    
} /* pciCanTransmitIsr */


/*======================================================================*/
/* Handle error interrupts. Happens when the bus status or error status */
/* bits in the status register changes.                                 */
/*======================================================================*/

static void pciCanErrorIsr (PciCanChanData *chd)
{
    pciCanRequestChipState (chd);
} /* pciCanErrorIsr */

/*======================================================================*/
/*  Overrun interrupt handler                                           */
/*======================================================================*/

static void pciCanOverrunIsr (PciCanChanData *chd)
{
    unsigned addr = chd->sja1000;
    outb (PCAN_CDO, addr + PCAN_CMR);
    chd->overrun = VCAN_MSG_FLAG_OVERRUN;
    pciCanReceiveIsr (chd);
} /* pciCanOverrunIsr */

/*======================================================================*/
/*  Bus error interrupt handler                                         */
/*======================================================================*/

static void pciCanBusErrorIsr (PciCanChanData *chd)
{
    unsigned circAddr = chd->sja1000;
    VCAN_EVENT e;
    unsigned char ECC;
    int r;
    
    ECC = inb(circAddr + PCAN_ECC);
    
    e.timeStamp = pciCanTime(chd->usPerTick);
    e.tag = V_RECEIVE_MSG;
    e.transId = 0;
    e.tagData.msg.id = 0x800 + ECC;
    e.tagData.msg.flags = VCAN_MSG_FLAG_ERROR_FRAME;
    e.tagData.msg.dlc = 0;
    
    r = pciCanDispatchEvent(chd, &e);
    
    //
    // Muffle the sja1000 if we get too many errors.
    //
    chd->errorCount++;
    if (chd->errorCount == MAX_ERROR_COUNT/2) {
        /* Half done, store current time */
        chd->errorTime = pciCanTime(10);
    }
    else if (chd->errorCount > MAX_ERROR_COUNT) {
        if ((pciCanTime(10) - chd->errorTime) > ERROR_RATE/10) {
            /* Error rate reasonable, restart counters */
            chd->errorCount = 0;
            chd->errorTime = pciCanTime(10);
        }
        else {
            unsigned char ier;
            ier = inb (circAddr + PCAN_IER);
            outb((unsigned char)(ier & ~PCAN_BEIE), circAddr + PCAN_IER);
        }
    }
} /* pciCanBusErrorIsr */


static void pciCanErrorPassiveIsr (PciCanChanData *chd)
{
    pciCanRequestChipState(chd);
} /* pciCanErrorPassiveIsr */

/*======================================================================*/
/*  Main ISR                                                            */
/*======================================================================*/

void pciCanInterrupt(int irq, void *dev_id, struct pt_regs *regs)
{
    PciCanCardData *cd = (PciCanCardData*) dev_id;
    PciCanChanData *chd;
    unsigned int loopmax = 1000;
    unsigned ireg;
    int i;
    
    /* Read interrupt status */
    while(inl(cd->pciIf + S5920_INTCSR) & INTCSR_INTERRUPT_ASSERTED_M){
        
        if (--loopmax == 0) {
            /* Kill the card. */
            printk ("<1> PCIcan runaway, shutting down card!!");
            pciCanResetCard (cd);
            return;
        }
        
        /* Handle all channels */
        for (i = 0; i < cd->nrChannels; i++) {
            chd = &cd->chanData[i];
            /* Reading clears interrupt flag */
            while ((ireg = inb(chd->sja1000 + PCAN_IR))) {
                if (ireg & PCAN_RI) pciCanReceiveIsr(chd);
                if (ireg & PCAN_TI) pciCanTransmitIsr(chd);
                if (ireg & PCAN_EI) pciCanErrorIsr(chd);
                if (ireg & PCAN_DOI) pciCanOverrunIsr(chd);
                if (ireg & PCAN_WUI) { 
                    printk ("<1>PCIcan: Huh? Wakeup Interrupt!\n");
                }
                if (ireg & PCAN_BEI) pciCanBusErrorIsr(chd);
                if (ireg & PCAN_EPI) pciCanErrorPassiveIsr(chd);
            }
        }
    }
    
} /* pciCanInterrupt */

/*======================================================================*/
/* Init data structures for one channel                                 */
/*======================================================================*/

static int pciCanInitChanData(PciCanChanData *chd)
{
    chd->minorNr = driverData.minorNr++;
    
    /* Init waitqueues */
    init_waitqueue_head (&(chd->flushQ));
    init_waitqueue_head (&(chd->txChanWaitQ));
    spin_lock_init(&(chd->openLock));
    spin_lock_init(&(chd->sendQlock));
    chd->txTaskQ.routine = pcicanSend;
    chd->txTaskQ.data = chd;
    chd->usPerTick = 1000;
    
    return 0;
}

/*======================================================================*/
/*  pcicanTransmit                                                      */
/*======================================================================*/

static int pciCanTransmitMessage (PciCanChanData *chd, CAN_MSG *m)
{
    unsigned       p;
    unsigned       circAddr = chd->sja1000;
    unsigned char  *msg = m->data;
    signed long    ident = m->id;
    unsigned char  flags = m->flags;
    unsigned char  dlc = m->length;
    int            i;
    
    /* Set special transvceiver modes */
    switch (chd->transType) {
        
    case VCAN_TRANSCEIVER_TYPE_SWC:
        if (flags & VCAN_MSG_FLAG_WAKEUP) {
            pciCanActivateTransceiver (chd, VCAN_TRANSCEIVER_LINEMODE_SWC_WAKEUP);
        }
        break;
        
    case VCAN_TRANSCEIVER_TYPE_251:
    case VCAN_TRANSCEIVER_TYPE_NONE:
        break;
        
    default:
        break;
    }
    
    if (ident & VCAN_EXT_MSG_ID) { /* Extended CAN */
        WL id;
        unsigned char x;
        
        id.L = ident & ~VCAN_EXT_MSG_ID;
        id.L <<= 3;
        x = (unsigned char)(dlc | PCAN_FF_EXTENDED);
        if (flags & VCAN_MSG_FLAG_REMOTE_FRAME) x |= PCAN_FF_REMOTE;
        
        outb(x, circAddr + PCAN_MSGBUF);
        outb(id.B.b3, circAddr + PCAN_XID0);
        outb(id.B.b2, circAddr + PCAN_XID1);
        outb(id.B.b1, circAddr + PCAN_XID2);
        outb(id.B.b0, circAddr + PCAN_XID3);
        
        p = circAddr + PCAN_XDATA;
        for (i = 0; i < dlc; i++) outb (*msg++, p++);
    }
    else { /* Standard CAN */
        unsigned char x;
        x = dlc;
        
        if (flags & VCAN_MSG_FLAG_REMOTE_FRAME) x |= PCAN_FF_REMOTE;
        outb(x, circAddr + PCAN_MSGBUF);
        outb((unsigned char) (ident >> 3), circAddr + PCAN_SID0);
        outb((unsigned char) (ident << 5), circAddr + PCAN_SID1);
        
        p = circAddr + PCAN_SDATA;
        for (i=0; i<dlc; i++) outb(*msg++, p++);
    }
    
    chd->currentTxMsg = m;
    
    if (flags & VCAN_MSG_FLAG_TXRQ) {
        VCAN_EVENT e = *((VCAN_EVENT*) chd->currentTxMsg);
        e.tagData.msg.flags &= ~(VCAN_MSG_FLAG_TX_NOTIFY | VCAN_MSG_FLAG_TX_START);
        
        /* fix invalid messages */
        e.tagData.msg.flags &= (VCAN_MSG_FLAG_ERROR_FRAME |
            VCAN_MSG_FLAG_REMOTE_FRAME);
        
        if (e.tagData.msg.dlc > 8) e.tagData.msg.dlc = 8;
        
        if (e.tagData.msg.id & VCAN_EXT_MSG_ID) {
            e.tagData.msg.id &= 0x1fffffff | VCAN_EXT_MSG_ID;
        }
        else {
            e.tagData.msg.id &= 0x07ff;
        }
        
        e.tag = V_RECEIVE_MSG;
        e.timeStamp = pciCanTime(chd->usPerTick);
        
        e.tagData.msg.flags |= VCAN_MSG_FLAG_TX_START;
        pciCanDispatchEvent(chd, &e);
    }
    
    
    outb(PCAN_TR, circAddr + PCAN_CMR);
    
    if(chd->errorCount > 0) pciCanResetErrorCounter (chd);
    
    return 0;
} /* pciCanTransmitMessage */



/*======================================================================*/
/*  Initialize CAN controller                                           */
/*======================================================================*/

static int pciCanInitChanHW (PciCanChanData *chd)
{
    unsigned addr;
    
    addr = chd->sja1000;
    if (!addr) return -1;
    
    // Reset the circuit...
    outb(PCAN_RM, addr + PCAN_MOD);
    // 
    // ...goto Pelican mode...
    outb(PCAN_PELICAN|PCAN_CBP, addr + PCAN_CDR);
    // 
    // ...and set the filter mode
    outb(PCAN_RM|PCAN_AFM, addr + PCAN_MOD);
    
    // Activate almost all interrupt sources.
    outb(PCAN_BEIE|PCAN_EPIE|PCAN_DOIE|PCAN_EIE|PCAN_TIE|PCAN_RIE, addr + PCAN_IER);
    
    // Accept all messages by default.
    
    outb(0, addr + PCAN_ACR0);
    outb(0, addr + PCAN_ACR1);
    outb(0, addr + PCAN_ACR2);
    outb(0, addr + PCAN_ACR3);
    outb(0xFF, addr + PCAN_AMR0);
    outb(0xFF, addr + PCAN_AMR1);
    outb(0xFF, addr + PCAN_AMR2);
    outb(0xFF, addr + PCAN_AMR3);
    
    // Default 125 kbit/s, pushpull.
    outb(0x07, addr + PCAN_BTR0);
    outb(0x23, addr + PCAN_BTR1);
    
    if (chd->transType == VCAN_TRANSCEIVER_TYPE_GAL) {
        outb(OCR_DEFAULT_GAL, addr + PCAN_OCR);
    } else {
        outb(OCR_DEFAULT_STD, addr + PCAN_OCR);
    }
    
    return 0;
} 

/*======================================================================*/
/*  Find out addresses for one card                                     */
/*======================================================================*/

static int scanPCIAddresses(struct pci_dev *dev, PciCanCardData *cardData)
{
    int i;
    
    static u32 addresses[] = {
        PCI_BASE_ADDRESS_0,
            PCI_BASE_ADDRESS_1,
            PCI_BASE_ADDRESS_2,
            PCI_BASE_ADDRESS_3,
            PCI_BASE_ADDRESS_4,
            PCI_BASE_ADDRESS_5,
            0
    };
    
    
    for(i = 0;addresses[i];i++) {
        u32 curr,mask;
        
        pci_read_config_dword(dev,addresses[i],&curr);
        cli();
        pci_write_config_dword(dev,addresses[i],~0);
        pci_read_config_dword(dev,addresses[i],&mask);
        pci_write_config_dword(dev,addresses[i],curr);
        sti();
        if(!mask)
            continue;
        
        
        if(curr & PCI_BASE_ADDRESS_SPACE_IO) {
            curr &= PCI_BASE_ADDRESS_IO_MASK;
        } else {
            curr &= PCI_BASE_ADDRESS_MEM_MASK;
        }
        
        if(mask & PCI_BASE_ADDRESS_SPACE_IO) {
            mask &= PCI_BASE_ADDRESS_IO_MASK;
        } else {
            mask &= PCI_BASE_ADDRESS_MEM_MASK;
        }
        
        
        switch(i){
        case 0:
            cardData->pciIf = curr;
            break;
        case 1:
            cardData->sjaBase = curr;
            cardData->irq = dev->irq;
            break;
        case 2:
            cardData->xilinx = curr;
            break;
        default:
            break;
        }      
    }
    /* we return 1 for ok */
    if (pci_enable_device(dev)){
        /* Failed */
        return 0;
    } else {
        return 1;
    }
    
}    

/*======================================================================*/
/* Initialize the HW for one card                                       */
/*======================================================================*/

int pciCanInitOne(struct pci_dev *dev)
{
    int ret;
    int i;
    PciCanCardData *cardData;
    
    /* Allocate data area for this card */
    cardData  = kmalloc(sizeof(PciCanCardData), GFP_KERNEL);
    if (!cardData) return 0;
    memset(cardData, 0, sizeof(PciCanCardData));
    
    /* Get PCI controller, SJA1000 base and Xilinx addresses */
    ret = scanPCIAddresses(dev, cardData);
    if (ret == 0) {
        kfree(cardData);
        printk("<1>scanPCIAddresses failed");
        return 0;
    }
    
    /* Find out type of card i.e. N/O channels etc */
    if (pciCanProbe(cardData)) {
        kfree(cardData);
        printk("<1>pciCanProbe failed");
        return 0;	
    }
    
    /* Init channels */
    for (i = 0; i < cardData->nrChannels; i++){
        PciCanChanData *chd = &cardData->chanData[i];
        
        DEBUG(1, "transType=%d\n", chd->transType);
        /* Init data structures for the channel */
        pciCanInitChanData(chd);
        pciCanInitChanHW(chd);
    }
    
    /* Enable interrupts in PCI Interface*/
    if (pciCanInitHW (cardData)) {
        kfree(cardData);
        printk("<1> pciCanInitHW failed");
        return 0;	
    }
    
    /* Insert into list of cards */
    cardData->next = pciCanCards;
    pciCanCards = cardData;
    /* ISR */
    request_irq(cardData->irq, pciCanInterrupt, SA_SHIRQ, "Kvaser PCIcan", cardData);
    
    return 1; 
} /* pciCanInitOne */


/*======================================================================*/
/* Find and initialize all cards                                        */
/*======================================================================*/

int pciCanFindAllDevices(void)
{
    struct pci_dev *dev = NULL;
    int found;
    
    if (!pci_present())
        return -ENODEV;
    
    for (found=0; found < PCICAN_MAX_DEV;) {
        dev = pci_find_device(PCICAN_VENDOR, PCICAN_ID, dev);
        if (!dev) /* No more PCIcans... */
            break;
        /* Initialize card */
        found += pciCanInitOne(dev);
    }
    /* We need to find at least one */
    return  (found == 0) ? -ENODEV : 0;
} /* pciCanFindAllDevices */

/*======================================================================*/
/* Shut down and free resources before unloading driver                 */
/*======================================================================*/

int pciCanCloseAllDevices(void)
{
    /* qqq check for open files */
    PciCanCardData *cardData;
    
    cardData = pciCanCards;
    while (cardData) {
        free_irq(cardData->irq, cardData);
        
        cardData = pciCanCards->next;
        kfree(pciCanCards);
        pciCanCards = cardData;
    }
    return 0;
} /* pciCanCloseAllDevices */

/*======================================================================*/
/* Module init                                                          */
/*======================================================================*/

/* Major device number qqq */  

int init_module(void)
{
    int result;
    DEBUG(1, "--------------------------------------------------\n");
    DEBUG(1, "%s\n", version);
    DEBUG(1, "< "__DATE__ " > < " __TIME__" >\n");
    DEBUG(1, "--------------------------------------------------\n");    
    /* Initialize card and data structures */
    if (pciCanFindAllDevices() == -ENODEV) {
        DEBUG(1, "No Kvaser PCIcan cards found!");
        return -1;
    }
    
    /* Register driver for device */
    //    driverData.majorDevNr = register_chrdev(majorDevNr, device_name, &pcicanFOps);
    driverData.majorDevNr = 0;
    result = register_chrdev(driverData.majorDevNr, device_name, &pcicanFOps);
    
    if (result < 0) {
        DEBUG(1,"register_chrdev(%d, %s, %x) failed, error = %d\n",
            driverData.majorDevNr, device_name, (int) &pcicanFOps, result);
        pciCanCloseAllDevices();
        return -1;
    }    
    if (driverData.majorDevNr == 0) driverData.majorDevNr = result;
    
    do_gettimeofday(&driverData.startTime);
    
    create_proc_read_entry("pcican", 
        0    /* default mode */,
        NULL /* parent dir */, 
        pciCanProcRead, 
        NULL /* client data */);
    
    return 0;
}

/*======================================================================*/
/* Module shutdown                                                      */
/*======================================================================*/

void cleanup_module(void)
{
    if (driverData.majorDevNr > 0){
        unregister_chrdev(driverData.majorDevNr, device_name);
    }
    remove_proc_entry("pcican", NULL /* parent dir */);
    pciCanCloseAllDevices();
}
/*======================================================================*/












